#include <aesbind.h> /* You'll need most of the these includes */
#include <vdibind.h>
#include <gemdefs.h>
#include <obdefs.h>
#include <string.h>
#include <stdio.h> /* These two are for file access only */
#include <stat.h>
#include "window.h" /* Of ourse this one's pretty important */
#include "windemo.h"

/* These are the toys we'll be using with our windows */
#define WINKIND \
NAME | CLOSER | MOVER | FULLER | SIZER | \
LFARROW | RTARROW | HSLIDE | \
UPARROW | DNARROW | VSLIDE

#define LINE_LEN 80 /* A good number for most text apps (maybe 132 is better) */

typedef struct text_win /* Kinda convoluted... seemed like a good idea */
{
  char *filename ;
  win_struct_ptr window ;
} text_win_struct ;

text_win_struct windows[] = 
{
  "window.lex" , NULL ,
  "window.h" , NULL ,
  "windemo.c" , NULL ,
  NULL , NULL 
} ;

char *realloc ( ) ; /* Sure wish MWC had come with stdlib.h */
char *lmalloc ( ) ; /* Guess I should type it in one day */

/* The customary GEM declarations... you have to declare these or else */
int contrl[12] , intin[128] , ptsin[128] , intout[128] , ptsout[128] ;

/* 
 * Now for the real meat and potatoes!!! This is a common redraw routine
 * for a window containing text. It's used by all three windows.
 */
void text_redraw ( win , cells , pixels )
register win_struct_ptr win ;
register box_struct *cells , *pixels ;
{
  /* For text I use the user pointer as an array of character pointers */
  char **text = ( char** ) win->user ; 
  pixels->xw += pixels->x - 1 ; /* convert pixels to VDI rectangle */
  pixels->yh += pixels->y - 1 ;
  vr_recfl ( win->vdi , pixels ) ; /* Clear the area we're redrawing */
  cells->x = 0 ; /* Text strings usually start at index 0 */
  if ( text ) /* If there's no text then there's no need to redraw */
  {

    /* Redraw the area on line at a time */
    for ( ; cells->yh >= 0 ; -- cells->yh , cells->y ++ )
    {

      /* It's possible that the view is larger than the amount of text */
      if ( ( cells->y < win->world.yh ) )
      {

        /* Calculate the top left pixel of the first character in the string */
        win_convert ( win , TO_PIXELS , cells , pixels ) ;
        /* Draw the string... remember clipping was done for us */
        v_gtext 
        ( win->vdi , pixels->x , pixels->y , text[cells->y] ) ;
      }
    }
  }
}

/*
 * I don't feel like commenting this routine since it has nothing to do with
 * windowing. Suffice it to say that you pass a filename and it returns a
 * dynamically allocated array of character pointers to each line of text in
 * the file and the number of lines of text in the array/file
 */
char **load_textfile ( filename , lines )
char *filename ;
int *lines ;
{
  struct stat ts ;
  FILE *tf ;
  char *td ;
  char **tl = NULL ;
  *lines = 0 ;
  stat ( filename , &ts ) ;
  if ( tf = fopen ( filename , "r" ) )
  {
    td = lmalloc ( ts.st_size + 1 ) ; 
    while ( fgets ( td , LINE_LEN + 1 , tf ) )
    {
      tl = realloc ( tl , ( ( *lines ) + 1 ) * sizeof ( char * ) ) ;
      tl[( *lines ) ++] = td ;
      td += strlen ( td ) - 1 ;
      if ( *td == '\n' )
      {
        *( td ++ ) = '\0' ;
      }
    }
    fclose ( tf ) ;
  }
  return ( tl ) ;
}

/* This preamble sets the parameters of the windows we're using */
void text_win_init ( win , filename )
win_struct_ptr win ;
char *filename ;
{
  static int work_in[] = { 1 , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , 1 , 2 } ;
  int work_out[57] ;
  int dummy , lines , width , height ;

  /* Get the vdi handle that AES is using (returned by v_opnwk ( ) ) */
  int vdi = graf_handle ( &dummy , &dummy , &dummy , &dummy ) ;
  win_init ( win ) ; /* This is essential before any other initialization */
  win_setkind ( win , WINKIND ) ; /* Select the toys for use with the window */
  win_setnameinfo ( win , filename , win->info ) ; /* Filename in name box */
  win_setlimit ( win , 40 , 10 , 80 , 25 ) ; /* Good limits for a text window */
  v_opnvwk ( work_in , &vdi , work_out ) ; /* Open  virtual workstation */

  /* When I use v_gtext ( ) I'll pass the upper left corner of the first char */
  vst_alignment ( vdi , 0 , 5 , &dummy , &dummy ) ;
  vst_point ( vdi , 0 , &dummy , &dummy , &width , &height ) ; /* Small font */
  win_setcellvdi ( win , width , height , vdi ) ; /* Cell size same as char */
  win_setredraw ( win , text_redraw ) ; /* Use generic text redraw routine */
  win->user = load_textfile ( filename , &lines ) ; /* Load the file */
  win_setworld ( win , 0 , 0 , LINE_LEN , lines ) ; /* World is size of file */
}

main ( )
{
  box_struct temp ;
  int msg[8] ;
  int i ;
  OBJECT *menu ;
  int done = 0 ;
  int aes = appl_init ( ) ; /* What else is new */
  if ( aes >= 0 )
  {
    rsrc_load ( "windemo.rsc" ) ; /* Load and draw the menu bar */
    rsrc_gaddr ( R_TREE , WINMENU , &menu ) ;
    menu_bar ( menu , 1 ) ;
    for ( i = 0 ; windows[i].filename ; i ++ ) /* Initialize the windows */
    {
      windows[i].window = ( win_struct_ptr ) malloc ( sizeof ( win_struct ) ) ;
      text_win_init ( windows[i].window , windows[i].filename ) ;
      win_sethome ( windows[i].window , 
      windows[i].window->home.x + ( i * windows[i].window->cell.x ) ,
      windows[i].window->home.y + ( i * windows[i].window->cell.y ) ) ;
    }
    graf_mouse ( 3 , NULL ) ; /* Get rid of the usy bee */
    do /* This is it.. the event loop */
    {
      evnt_mesag ( msg ) ; /* Wait for a message */
      if ( msg[0] == MN_SELECTED ) /* Was it a menu selection */
      {
        switch ( msg[4] )
        {
          case ABOUT : /* So you want to know more */
          {
            form_alert ( 1 , 
            "[1][Stainless Windows Toolbox demo| |Dave Steele (Aug-91)][ OK ]"
            ) ;
            break ;
          }
          case QUIT : /* Finished for now */
          {
            done = 1 ;
            break ;
          }
          default : /* Open or top a window */
          {

            /* This makes the window explode open from the menu entry */
            memcpy 
            ( &temp.xw , &(menu[msg[4]].ob_width) , sizeof ( point_struct ) ) ;
            objc_offset ( menu , msg[4] , &temp.x , &temp.y ) ;
            win_open ( windows[msg[4] - FIRSTWIN].window , &temp ) ;
            break ;
          }
        }
        menu_tnormal ( menu , msg[3] , 1 ) ; /* Clear the highlighted entry */
      }
      else
      {

        /* If it's a window message then handle it.. no muss no fuss */
        window_msg ( msg ) ; 
      }
    }
    while ( ! done ) ; /* Someone must have selected quit */
    for ( i = 0 ; windows[i].filename ; i ++ ) /* Close windows, free memory */
    {
      if ( windows[i].window->user )
      {
        free ( ( ( char ** ) windows[i].window->user )[0] ) ;
        free ( windows[i].window->user ) ;
      }
      v_clsvwk ( windows[i].window->vdi ) ;
      win_close ( windows[i].window , NULL ) ;
      free ( windows[i].window ) ;
    }
    appl_exit ( ) ; /* Exit application */
  }
}
