/* flags used by win_convert function */
enum
{
  TO_CELLS ,
  TO_PIXELS
} ;

/* cartesian co-ordinate point */
typedef struct point
{
  int x , y ;
} point_struct ;

/* dual purpose AES or VDI box */
typedef struct box
{
  int x , y , xw , yh ;
} box_struct ;

/* everything you need to know about a window... and more */
typedef struct win
{
  int vdi ;             /* VDI handle                                   */
  box_struct world ;    /* origin and size of world in cells            */
  box_struct view ;     /* origin and size of view in cells             */
  point_struct cell ;   /* size of one cell                             */
  box_struct constrain ;/* pixel constraints of the window              */
  box_struct limit ;    /* min and max size of view in cells            */
  point_struct home ;   /* pixel origin of view window                  */
  int kind ;            /* AES window attributes                        */
  char *name ;          /* name pointer passed to wind_set when opened  */
  char *info ;          /* info pointer passed to wind_set when opened  */
  int handle ;          /* AES window handle or -1 if closed            */
  void ( *redraw ) ( ) ;/* user supplied redraw function                */
  int ( *prehook ) ( ) ;/* allows user to intercept window controls     */
  char *user ;          /* a free pointer for the user                  */
  point_struct oldsize ;/* used in full and restore operations          */
  struct winrec *next ; /* used to maintain linked-list of open windows */
} win_struct ;

/* pointer type to the latter struct */
typedef win_struct *win_struct_ptr ;

/* window function bindings */

void win_convert ( ) ;
/*
void win_convert ( win , type , frombox , tobox )
win_struct_ptr win ;
int type ;
box_struct *frombox , *tobox ;
*/

void win_init ( ) ;
/*
void win_init ( win )
win_struct_ptr win ;
*/

int win_open ( ) ;
/*
int win_open ( win , frombox )
win_struct_ptr win ;
box_struct *frombox ;
*/

void win_close ( ) ;
/*
void win_close ( win , tobox )
win_struct_ptr win ;
box_struct *tobox ;
*/

void win_cellshift ( ) ;
/*
void win_cellshift ( win , cells , dx , dy )
win_struct_ptr win ;
box_struct *cells ;
int dx , dy ;
*/

void win_shift ( ) ;
/*
void win_shift ( win , dx , dy )
win_struct_ptr win ;
int dx , dy ;
*/

void win_setkind ( ) ;
/*
void win_setkind ( win , kind )
win_struct_ptr win ;
int kind ;
*/

void win_setnameinfo ( ) ;
/*
void win_setnameinfo ( win , name , info )
win_struct_ptr win ;
char *name , *info ;
*/

void win_setconstrain ( ) ;
/*
void win_setconstrain ( win , x , y , w , h )
win_struct_ptr win ;
int x , y , w , h ;
*/

void win_sethome ( ) ;
/*
void win_sethome ( win , x , y )
win_struct_ptr win ;
int x , y ;
*/

void win_setlimit ( ) ;
/*
void win_setlimit ( win , xmin , ymin , xmax , ymax )
win_struct_ptr win ;
int xmin , ymin , xmax , ymax ;
*/

void win_setworld ( ) ;
/*
void win_setworld ( win , x , y , w , h )
win_struct_ptr win ;
int x , y , w , h ;
*/

void win_setview ( ) ;
/*
void win_setview ( win , x , y , w , h )
win_struct_ptr win ;
int x , y , w , h ;
*/

void win_setcellvdi ( ) ;
/*
void win_setcellvdi ( win , x , y , vdi )
win_struct_ptr win ;
int x , y , vdi ;
*/

void win_setredraw ( ) ;
/*
void win_setredraw ( win , redraw )
win_struct_ptr win ;
void ( *redraw ) ( ) ;
*/

void win_setprehook ( ) ;
/*
void win_setprehook ( win , prehook )
win_struct_ptr win ;
int ( *prehook ) ( ) ;
*/

win_struct_ptr win_getrec ( ) ;
/*
win_struct_ptr win_getrec ( handle )
int handle ;
*/

void win_normal ( ) ;
/*
void win_normal ( win )
win_struct_ptr win ;
*/

int window_msg ( ) ;
/*
int window_msg ( msg )
int msg[8] ;
*/

/* templates for user supplied functions */

/*
void redraw ( win , cellrect , pixrect )
win_struct_ptr win ;
box_struct *cellrect , *pixrect ;
*/

/*
int prehook ( win , msg )
win_struct_ptr win ;
int msg[8] ;
*/
